@@ -102,6 +102,15 @@ type solution = preamble * universe
     the given package list *)
 val load_universe : package list -> universe
 
+(** add a package to an existing universe. The universe is modified in place.
+    @raise Constraint_violation if a package with the same name and version is
+    alreayd in the given universe *)
+val add_package : universe -> package -> unit
+
+(** remove a package from an existing universe.
+    The universe is modified in place *)
+val remove_package : universe -> pkgname * version -> unit
+
 (** {5 CUDF manipulation} *)
 
 (** Lookup a specific package via a <name, version> key
@@ -166,6 +175,17 @@ val fold_packages : ('a -> package -> 'a) -> 'a -> universe -> 'a
     both the package and its unique identifier *)
 val iteri_packages : (int -> package -> unit) -> universe -> unit
 
+(** iter on all packages grouped by name. Each package name is associated to
+    a list of packages with the same name and different versions *)
+val iter_packages_by_name : (pkgname -> package list -> unit) -> universe -> unit
+
+(** fold on all packages grouped by name. Each package name is associated to
+    a list of packages with the same name and different versions *)
+val fold_packages_by_name : ('a -> pkgname -> package list -> 'a) -> 'a -> universe -> 'a
+
+(** return the list of all unique package names *)
+val package_names : universe -> pkgname list
+
 (** conversion from universe to plain package list
 
     @param filter only return packages matching a given
